@@ -1,12 +1,13 @@
 swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rho=0, gamma=0, zeta=0, icc=0, cac=1, iac=0,
-                         alpha = 0.05, retDATA = FALSE, silent = FALSE)
+                         ar=1, alpha = 0.05, retDATA = FALSE, silent = FALSE)
 {
-  #Version: 4/28/2023, v. 4.0, Jim Hughes
+  # Version: 11/4/2024, v. 4.1, Jim Hughes
   # Note: zeta = 0 and iac = 0 means function defaults to a cross-sectional design; either zeta >0 or iac>0 means a closed cohort design
   ##### Helper functions #######
   replaceNAwithZero = function(x){
     ifelse(is.na(x),0,x)
   }
+  is.scalar <- function(x) is.atomic(x) && length(x) == 1L
   ##########
   #Warnings
   ##########
@@ -23,7 +24,7 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
   }
   if(!is.null(H)){
     if (design$nTxLev > 1){
-    stop("Do not use H when you have specfied a design with multiple treatment levels. If you want an ETI model based estimator, specify a 0/1 intervention in swDsn or swDsn1 and use H to define the estimator.")
+    stop("Do not use H when you have specfied a design with multiple treatment levels. If you want an ETI model based estimator, specify a 0/1 intervention in swDsn and use H to define the estimator.")
     }
     if (any(design$tx.effect.frac!=1)){
       stop("Do not specify fractional treatment effects in the design AND an ETI estimator")
@@ -42,15 +43,18 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
 	  }
   }
   if(design$nTxLev!=length(mu1)){
-    stop("Length of mu1 must correspond to number of treatment levels specified in swDsn1")
+    stop("Length of mu1 must correspond to number of treatment levels specified in swDsn")
   }
   if (distn == 'gaussian' & missing(sigma)) stop("If distribution is gaussian, a value for sigma must be entered")
   #Checks to make sure people are using random effects OR ICC/CAC/IAC:
   param.icc.any <- !missing(icc) | !missing(cac) | !missing(iac)
-  param.re.any <- !missing(tau) | !missing(eta) | !missing(rho) | !missing(gamma) | !missing(zeta)
+  param.re.any <- !missing(tau) | !missing(eta) | !missing(rho) | !missing(gamma) | !missing(zeta) 
   if(param.re.any == TRUE & param.icc.any == TRUE){
     stop("The two parameterizations (random effects and ICC/CAC/IAC) are mutually exclusive.  Either enter values for ICC, CAC (and IAC, if cohort design), or tau, eta, gamma, rho (and zeta, if cohort design).")
   }
+  if (ar!=1 & (zeta!=0 | iac!=0)) {
+    stop("autoregressive structure (ar!=1) only allowed with cross-sectional sampling")
+  }
   ##########
   #If using ICC/CAC/IAC, translate to random effects
   ##########
@@ -92,22 +96,31 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
   #More warnings for variance components
   ##########
   #Basic restrictions on variance components
-  if(rho < -1 | rho > 1){
+  if(any(rho < -1 | rho > 1)){
     stop("rho must be a numeral between -1 and 1.")
   }
-  if(tau < 0 | eta < 0 | gamma < 0 | zeta < 0){
-    stop("tau, eta, gamma and zeta must be non-negative numerals.")
-  }
-  if((tau == 0 | eta == 0) & rho != 0){
-    stop("If either tau or eta is zero, rho must be zero, since fixed effects cannot be correlated.")
+  if (is.scalar(eta)){
+      if (length(rho)>1) stop("eta is scalar so rho must be scalar")
+      if(tau < 0 | eta<0 | gamma < 0 | zeta < 0) stop("tau, eta, gamma, and zeta must be non-negative.")
+      if ((tau == 0 | eta == 0) & rho != 0) stop("If either tau or eta is zero, rho must be zero.")
   }
+  if (is.list(eta)){
+      if (is.scalar(rho)) rho = rep(rho, length(eta$eta))
+      if(tau < 0 | (any(eta$eta<0)) | gamma < 0 | zeta < 0) stop("tau, eta, gamma, and zeta must be non-negative.")
+      if (any((tau==0 | eta$eta==0) & rho!=0)) stop("If either tau or eta is zero, corresponding rho must be zero.")
+      if (length(eta$eta)!=design$nTxLev) stop("eta must be scalar or list with elements corresponding to number of treatment levels")
+      if (length(rho)!=design$nTxLev) stop("rho must be scalar or vector with length equal to number of treatment levels.") 
+      }
+  if(ar < -1 | ar > 1){
+    stop("ar must be a numeral between -1 and 1.")
+  }  
   if(distn == 'gaussian' && sigma == 0 & gamma == 0 & zeta == 0){
     stop("For a non-deterministic Gaussian outcome, at least one of sigma, zeta or gamma needs to be non-zero.")
   }
-  if (length(tau) > 1 | length(eta) > 1 | length(zeta) > 1)
-    stop("Function cannot compute stepped wedge design Power for tau-vector, zeta-vector or eta-vector; tau, zeta and eta must be scalars.")  ##########
+  if (length(tau) > 1 |  length(zeta) > 1 | length(ar) > 1)
+    stop("Function cannot compute stepped wedge design power for tau-vector, zeta-vector, or ar-vector; tau, zeta, ar must be scalars")
+    ##########
   ##########
-  obs.per.cluster.per.time <- n
   if (length(n)>1 & silent == FALSE){
     warning("When sample sizes are not uniform, power depends on order of clusters (see documentation).")
   }
@@ -130,8 +143,14 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
     if (!missing(sigma)& silent==FALSE) warning("sigma is not used when distn=binomial")
     sigSq <- muBar * (1 - muBar)
     sigma <- NA
-    if ((tau^2 + eta^2 + gamma^2) > sigSq)
+    if (is.scalar(eta)){
+      if ((tau^2 + eta^2 + gamma^2) > sigSq)
       stop("tau^2 + eta^2 + gamma^2 must be less than muBar*(1-muBar) when distn=binomial")
+    }
+    if (is.list(eta)){
+      if (any((tau^2 + eta$eta^2 + gamma^2) > sigSq))
+        stop("tau^2 + eta^2 + gamma^2 must be less than muBar*(1-muBar) when distn=binomial")
+    }
   }
 #########
   I.rep <- design$clusters
@@ -175,8 +194,9 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
   ##########
   #Make nMat, a matrix with a sample size for every cluster (rows) and time period (columns).
   if (length(n) == 1){
-    Wmat.blk <- (tau^2+zeta^2/n) + diag(sigSq/n+gamma^2, J)#matrix V_i
+    Wmat.blk <- (tau^2+zeta^2/n) + diag(gamma^2, J)#matrix V_i
     Wmat.partial <- kronecker(diag(1, I), Wmat.blk)#matrix V
+    Sigmat <- kronecker(diag(1, I),diag(sigSq/n, J))
     #above, I is the number of times that Wmat.blk gets repeated along the diagonal of a matrix filled with 0's otherwise
   }else{
     #Took this nMat creation from swSim: each row is for a specific cluster
@@ -194,36 +214,58 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
       }
       nMat <- n#
     }
-    Wmat.partial <- matrix(0,nrow=I*J,ncol=I*J)#Wmat.blk doesn't get used anywhere else, so just setting up the Wmat.partial
+    Wmat.partial <- matrix(0,nrow=I*J,ncol=I*J)
+    Sigmat <- matrix(0,nrow=I*J,ncol=I*J)
     for(i in 1:I){#for each of the I total clusters...
 #remember that sample size must be constant within a cluster if zeta > 0
-      Wmat.partial.i <- (tau^2 + zeta^2/sqrt(t(nMat[i,,drop=F])%*%nMat[i,])) + diag(sigSq/nMat[i,]) + diag(gamma^2,length(nMat[i,]))#Wmat.partial for cluster i
+      Wmat.partial.i <- (tau^2 + zeta^2/sqrt(t(nMat[i,,drop=F])%*%nMat[i,])) + diag(gamma^2,length(nMat[i,]))#Wmat.partial for cluster i
+      Sigmat.i <- diag(sigSq/nMat[i,])
       kronecker.diag.i <- rep(0,I)
       kronecker.diag.i[i] <- 1
       addition.i <- kronecker(diag(kronecker.diag.i), Wmat.partial.i)#Block diagonal matrix with the entries for block i
+      saddition.i <- kronecker(diag(kronecker.diag.i), Sigmat.i)
       #For a design with transition periods, need to clean out NaN's produced by multiplying 0*Inf (really want them to be 0)
       if(is.matrix(n)&any(n==0)){
-        addition.i[is.nan(addition.i)] <- 0
+        addition.i[is.nan(addition.i)|is.nan(saddition.i)] <- 0
+        saddition.i[is.nan(addition.i)|is.nan(saddition.i)] <- 0
       }
       Wmat.partial <- Wmat.partial + addition.i#For each cluster, we fill in another section of the block diagonal matrix
+      Sigmat <- Sigmat + saddition.i
     }
   }
   #
   #Make a matrix with the random effects that vary by treatment (eta, rho) in the correct locations, and add it to the other random effects.
   Xij.Xil.ARRAY <- array(NA, c(J, J, length(I.rep)))
-  for (i.indx in 1:length(I.rep)) {
-    Xi.jl <- as.numeric(swDesignUnique[i.indx, ] > 0)
-    Xij.Xil.ARRAY[, , i.indx] <- (Xi.jl %o% Xi.jl) * eta^2 + outer(Xi.jl, Xi.jl, "+") * rho * eta * tau
+  if (is.scalar(eta)){
+  # eta is a scalar
+    for (i in 1:length(I.rep)) {
+      Xi.jl <- as.numeric(swDesignUnique[i, ] > 0)
+      Xij.Xil.ARRAY[, , i] <- (Xi.jl %o% Xi.jl) * eta^2 + outer(Xi.jl, Xi.jl, "+") * rho * eta * tau
+    }
+  } else {
+  # eta is a list
+    for (i in 1:length(I.rep)) {
+      Xi.jl <- as.numeric(swDesignUnique[i, ] > 0)
+      tx <- replaceNAwithZero(swDesignUnique[i, ])
+      for (j in 1:J){
+        for (l in 1:J){
+          Xij.Xil.ARRAY[j,l,i] <- ifelse(Xi.jl[j]*Xi.jl[l]==1,eta$eta[tx[j]]*eta$eta[tx[l]]*eta$corr[tx[j],tx[l]],0) + 
+                                  ifelse(Xi.jl[j]==1,rho[tx[j]] * tau * eta$eta[tx[j]],0) + 
+                                  ifelse(Xi.jl[l]==1,rho[tx[l]] * tau * eta$eta[tx[l]],0)
+        }
+      }
+    }
   }
   Xij.Xil.LIST_blk <- sapply(1:length(I.rep), function(x) NULL)
-  for (i.indx in 1:length(I.rep)) {
-    Xij.Xil.LIST_blk[[i.indx]] <- kronecker(diag(1, I.rep[i.indx]), Xij.Xil.ARRAY[, , i.indx])
+  for (i in 1:length(I.rep)) {
+    Xij.Xil.LIST_blk[[i]] <- kronecker(diag(1, I.rep[i]), Xij.Xil.ARRAY[, , i])
   }
   W.eta <- blkDiag(Xij.Xil.LIST_blk)
-  Wmat <- Wmat.partial + as.matrix(W.eta)#Covariance matrix
- 
-  #Wmat has entries of Inf in rows/columns that correspond to timexcluster periods with no observations.
-  #Since these contain no information for the study, we can just remove them, and also adjust Xmat
+  #
+#  Wmat <- Wmat.partial*kronecker(diag(1,I), toeplitz(ar^seq(0,J-1),r=NULL)) + as.matrix(W.eta) + Sigmat#Covariance matrix
+  Wmat <- Wmat.partial*kronecker(diag(1,I), toeplitz(ar^seq(0,J-1))) + as.matrix(W.eta) + Sigmat#Covariance matrix
+#Wmat has entries of Inf in rows/columns that correspond to timexcluster periods with no observations.
+#Since these contain no information for the study, we can just remove them, and also adjust Xmat
   NAtimes <- is.na(as.vector(t(swDesign)))
   if((is.matrix(n)&any(n==0))|sum(NAtimes)>0){
     indices <- (rowSums(replaceNAwithZero(Wmat))==0) | NAtimes | (is.matrix(n) & as.vector(t(n)==0))
@@ -233,16 +275,36 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
   ##########
   #Use design matrix and covariance matrix to calculate power
   ##########
-
+  np <- ncol(Xmat)
+  tmp = t(Xmat) %*% solve(Wmat) %*% Xmat
+  pdrop = (1:np)[apply(tmp==0,1,sum)==np]
   if (is.null(H)) { 
   # IT model
-    np <- ncol(Xmat)
-    var <- solve(t(Xmat) %*% solve(Wmat) %*% Xmat)[(np-K+1):np,(np-K+1):np,drop=F]
+    if (length(pdrop)>0) {
+      if (any(pdrop<=np-K)) message("Time parameters ",pdrop[pdrop<=np-K]," not estimable and deleted from model\n")
+      if (any(pdrop>np-K)) {
+        message("Treatment parameters ",pdrop[pdrop>np-K] - (np-K)," not estimable and deleted from model\n")
+        K = K - sum(pdrop>(np-K))
+      }
+      np = np - length(pdrop)
+      tmp = tmp[-pdrop,-pdrop]
+      }
+    var <- solve(tmp)[(np-K+1):np,(np-K+1):np,drop=F]
     var.theta.WLS <- diag(var)
 	} else {
   # ETI model
-    eti = (ncol(Xmat)-maxET+1):ncol(Xmat)
-    var <- solve(t(Xmat) %*% solve(Wmat) %*% Xmat)[eti,eti]
+    if (length(pdrop)>0) {
+      if (any(pdrop<=np-maxET)) message("Time parameters ",pdrop[pdrop<=np-maxET]," not estimable and deleted from model\n")
+      if (any(pdrop>np-maxET)) {
+        message("Exposure time parameters ",pdrop[pdrop>np-maxET] - (np-maxET)," not estimable and deleted from model. If necessary, H has been renormalized\n")
+        H = H[-(pdrop[pdrop>np-maxET] - (np-maxET))]
+        H = H/sum(H)        
+        maxET = maxET - sum(pdrop>(np-maxET))
+      }
+      np = np - length(pdrop)
+      tmp = tmp[-pdrop,-pdrop]
+      }    
+    var <- solve(tmp)[(np-maxET+1):np,(np-maxET+1):np]
     var.theta.WLS <- t(as.matrix(H))%*%var%*%as.matrix(H)
     }
   pwrWLS <- pnorm(abs(theta)/sqrt(var.theta.WLS) - qnorm(1 -alpha/2)) + pnorm(-abs(theta)/sqrt(var.theta.WLS) - qnorm(1 -alpha/2))
@@ -281,7 +343,7 @@ swPwr <- function (design, distn, n, mu0, mu1, H = NULL, sigma, tau=0, eta=0, rh
   ##########
   if (retDATA)
     rslt <- list(design = design, n = n, mu0 = mu0, mu1 = mu1, H=H,
-                 tau = tau, eta = eta, rho = rho, sigma = sigma, gamma=gamma, zeta=zeta, alpha = alpha,
+                 tau = tau, eta = eta, rho = rho, sigma = sigma, gamma=gamma, zeta=zeta, ar = ar, alpha = alpha,
                  Xmat = Xmat, Wmat = Wmat, var=var, var.theta.WLS = var.theta.WLS,
                  pwrWLS = pwrWLS)
   rslt
